@@ -4,12 +4,21 @@ from gym import spaces
 from . import go
 from . import coords
 import numpy as np
+import os
+import pygame
 from pettingzoo.utils import wrappers
 
 
+def get_image(path):
+    import pygame
+    from os import path as os_path
+    cwd = os_path.dirname(__file__)
+    image = pygame.image.load(cwd + '/' + path)
+    return image
+
+
 def env(**kwargs):
     env = raw_env(**kwargs)
-    env = wrappers.CaptureStdoutWrapper(env)
     env = wrappers.TerminateIllegalWrapper(env, illegal_reward=-1)
     env = wrappers.AssertOutOfBoundsWrapper(env)
     env = wrappers.OrderEnforcingWrapper(env)
@@ -18,7 +27,7 @@ def env(**kwargs):
 
 class raw_env(AECEnv):
 
-    metadata = {'render.modes': ['human'], "name": "go_v2"}
+    metadata = {'render.modes': ['human', 'rgb_array'], "name": "go_v4"}
 
     def __init__(self, board_size: int = 19, komi: float = 7.5):
         # board_size: a int, representing the board size (board has a board_size x board_size shape)
@@ -32,6 +41,8 @@ class raw_env(AECEnv):
         self.possible_agents = self.agents[:]
         self.has_reset = False
 
+        self.screen = None
+
         self.observation_spaces = self._convert_to_dict(
             [spaces.Dict({'observation': spaces.Box(low=0, high=1, shape=(self._N, self._N, 3), dtype=np.bool),
                           'action_mask': spaces.Box(low=0, high=1, shape=((self._N * self._N) + 1,), dtype=np.int8)})
@@ -60,7 +71,7 @@ class raw_env(AECEnv):
             return np.ones([self._N, self._N], dtype=np.bool)
 
     def _encode_board_planes(self, agent):
-        agent_factor = -1 if agent == self.possible_agents[0] else 1
+        agent_factor = go.BLACK if agent == self.possible_agents[0] else go.WHITE
         current_agent_plane_idx = np.where(self._go.board == agent_factor)
         opponent_agent_plane_idx = np.where(self._go.board == -agent_factor)
         current_agent_plane = np.zeros([self._N, self._N], dtype=np.bool)
@@ -126,7 +137,73 @@ class raw_env(AECEnv):
         self._last_obs = self.observe(self.agents[0])
 
     def render(self, mode='human'):
-        print(self._go)
+        screen_width = 1026
+        screen_height = 1026
+
+        if self.screen is None:
+            if mode == "human":
+                pygame.init()
+                self.screen = pygame.display.set_mode((screen_width, screen_height))
+            else:
+                self.screen = pygame.Surface((screen_width, screen_height))
+        if mode == "human":
+            pygame.event.get()
+
+        size = go.N
+
+        # Load and scale all of the necessary images
+        tile_size = (screen_width) / size
+
+        black_stone = get_image(os.path.join('img', 'GoBlackPiece.png'))
+        black_stone = pygame.transform.scale(black_stone, (int(tile_size * (5 / 6)), int(tile_size * (5 / 6))))
+
+        white_stone = get_image(os.path.join('img', 'GoWhitePiece.png'))
+        white_stone = pygame.transform.scale(white_stone, (int(tile_size * (5 / 6)), int(tile_size * (5 / 6))))
+
+        tile_img = get_image(os.path.join('img', 'GO_Tile0.png'))
+        tile_img = pygame.transform.scale(tile_img, ((int(tile_size * (7 / 6))), int(tile_size * (7 / 6))))
+
+        # blit board tiles
+        for i in range(1, size - 1):
+            for j in range(1, size - 1):
+                self.screen.blit(tile_img, ((i * (tile_size)), int(j) * (tile_size)))
+
+        for i in range(1, 9):
+            tile_img = get_image(os.path.join('img', 'GO_Tile' + str(i) + '.png'))
+            tile_img = pygame.transform.scale(tile_img, ((int(tile_size * (7 / 6))), int(tile_size * (7 / 6))))
+            for j in range(1, size - 1):
+                if i == 1:
+                    self.screen.blit(tile_img, (0, int(j) * (tile_size)))
+                elif i == 2:
+                    self.screen.blit(tile_img, ((int(j) * (tile_size)), 0))
+                elif i == 3:
+                    self.screen.blit(tile_img, ((size - 1) * (tile_size), int(j) * (tile_size)))
+                elif i == 4:
+                    self.screen.blit(tile_img, ((int(j) * (tile_size)), (size - 1) * (tile_size)))
+            if i == 5:
+                self.screen.blit(tile_img, (0, 0))
+            elif i == 6:
+                self.screen.blit(tile_img, ((size - 1) * (tile_size), 0))
+            elif i == 7:
+                self.screen.blit(tile_img, ((size - 1) * (tile_size), (size - 1) * (tile_size)))
+            elif i == 8:
+                self.screen.blit(tile_img, (0, (size - 1) * (tile_size)))
+
+        offset = tile_size * (1 / 6)
+        # Blit the necessary chips and their positions
+        for i in range(0, size):
+            for j in range(0, size):
+                if self._go.board[i][j] == -1:
+                    self.screen.blit(black_stone, ((i * (tile_size) + offset), int(j) * (tile_size) + offset))
+                elif self._go.board[i][j] == 1:
+                    self.screen.blit(white_stone, ((i * (tile_size) + offset), int(j) * (tile_size) + offset))
+
+        if mode == "human":
+            pygame.display.update()
+
+        observation = np.array(pygame.surfarray.pixels3d(self.screen))
+
+        return np.transpose(observation, axes=(1, 0, 2)) if mode == "rgb_array" else None
 
     def close(self):
         pass
