@@ -14,6 +14,7 @@ class RLCardBase(AECEnv):
         super().__init__()
         self.name = name
         self.env = rlcard.make(name)
+        self.screen = None
         if not hasattr(self, "agents"):
             self.agents = [f'player_{i}' for i in range(num_players)]
         self.possible_agents = self.agents[:]
@@ -26,7 +27,10 @@ class RLCardBase(AECEnv):
         else:
             self._dtype = dtype
 
-        self.observation_spaces = self._convert_to_dict([spaces.Box(low=0.0, high=1.0, shape=obs_shape, dtype=self._dtype) for _ in range(self.num_agents)])
+        self.observation_spaces = self._convert_to_dict(
+            [spaces.Dict({'observation': spaces.Box(low=0.0, high=1.0, shape=obs_shape, dtype=self._dtype),
+                          'action_mask': spaces.Box(low=0, high=1, shape=(self.env.game.get_action_num(),),
+                                                    dtype=np.int8)}) for _ in range(self.num_agents)])
         self.action_spaces = self._convert_to_dict([spaces.Discrete(self.env.game.get_action_num()) for _ in range(self.num_agents)])
 
     def seed(self, seed=None):
@@ -46,7 +50,14 @@ class RLCardBase(AECEnv):
 
     def observe(self, agent):
         obs = self.env.get_state(self._name_to_int(agent))
-        return obs['obs'].astype(self._dtype)
+        observation = obs['obs'].astype(self._dtype)
+
+        legal_moves = self.next_legal_moves
+        action_mask = np.zeros(self.env.game.get_action_num(), int)
+        for i in legal_moves:
+            action_mask[i] = 1
+
+        return {'observation': observation, 'action_mask': action_mask}
 
     def step(self, action):
         if self.dones[self.agent_selection]:
@@ -56,10 +67,10 @@ class RLCardBase(AECEnv):
         self._last_obs = self.observe(self.agent_selection)
         if self.env.is_over():
             self.rewards = self._convert_to_dict(self._scale_rewards(self.env.get_payoffs()))
-            self.infos[next_player]['legal_moves'] = []
+            self.next_legal_moves = []
             self.dones = self._convert_to_dict([True if self.env.is_over() else False for _ in range(self.num_agents)])
         else:
-            self.infos[next_player]['legal_moves'] = list(sorted(obs['legal_actions']))
+            self.next_legal_moves = obs['legal_actions']
         self._cumulative_rewards[self.agent_selection] = 0
         self.agent_selection = next_player
         self._accumulate_rewards()
@@ -73,7 +84,7 @@ class RLCardBase(AECEnv):
         self._cumulative_rewards = self._convert_to_dict([0 for _ in range(self.num_agents)])
         self.dones = self._convert_to_dict([False for _ in range(self.num_agents)])
         self.infos = self._convert_to_dict([{'legal_moves': []} for _ in range(self.num_agents)])
-        self.infos[self._int_to_name(player_id)]['legal_moves'] = list(sorted(obs['legal_actions']))
+        self.next_legal_moves = list(sorted(obs['legal_actions']))
         self._last_obs = obs['obs']
 
     def render(self, mode='human'):
